//------------------------------------------------------------------------------
// Copyright (C) 2010, Raditex AB
// All rights reserved.
//
// FreeSCADA 
// http://www.FreeSCADA.com
// freescada@freescada.com
//
//------------------------------------------------------------------------------

#include <stdio.h>
#include <string.h>

#include "modbus.h"
#include "modbus-udp.h"


//------------------------------------------------------------------------------
// Execution starts here:
//------------------------------------------------------------------------------
int
main(int argc, char **argv)
{
    static char *func_name = "modbus_udp_dac", *host;

    int port = 502;
    int addr = 0, value, debug = 0;
        
    modbus_packet_t     *pkt;    
    modbus_udp_handle_t handle;

    if (argc < 4 || ((strcmp(argv[3], "read") != 0)))
    {
        printf("%s: usage: %s host addr command [value]\n", func_name, argv[0]);
        printf("\taddr    =  1,2,3,4\n");
        printf("\tcommand = read | write value (value = 0 - 255)\n");
        return 0;
    }
 
    host = argv[1];   
    addr = atoi(argv[2]) - 1;

    if (modbus_udp_init(host, port, &handle) != 0)
    {
        printf("%s: modbus_udp_init failed.\n", __PRETTY_FUNCTION__);
        return 0;
    }
    
    // setup request packet
    pkt = modbus_packet_new();  

    modbus_read_holding_registers(pkt, addr+4, 0x0001);

    if (debug)
    {
        modbus_packet_print(pkt);
    }

    // Send command
    if (modbus_udp_send(&handle, pkt) != 0)
    {
        printf("%s: modbus_udp_send failed.\n", __PRETTY_FUNCTION__);
        return 0;       
    }

    // recv response
    if (modbus_udp_recv(&handle, pkt) != 0)
    {
        printf("%s: modbus_udp_recv failed.\n", __PRETTY_FUNCTION__);
        return 0;       
    }
   
    if (debug)
    {
        modbus_packet_print(pkt);
    }

    value = modbus_packet_reg16_get(pkt, 0);

    printf("ADC %d = %d\n", addr, value);

    if (debug)
    {
        printf("\n");
        printf("Response byte 1 0x%.2X\n", pkt->data_buff[0] & 0xFF);    
        printf("Response byte 2 0x%.2X\n", pkt->data_buff[1] & 0xFF);    
        printf("Response byte 3 0x%.2X\n", pkt->data_buff[2] & 0xFF);    
    }
    

    modbus_packet_free(pkt);

    return 0;
}



