//------------------------------------------------------------------------------
// Copyright (C) 2010, Raditex AB
// All rights reserved.
//
// FreeSCADA 
// http://www.FreeSCADA.com
// freescada@freescada.com
//
//------------------------------------------------------------------------------

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <glib.h>

#include "modbus.h"
#include "modbus-udp.h"


//------------------------------------------------------------------------------
// Execution starts here:
//------------------------------------------------------------------------------
int
main(int argc, char **argv)
{
    static gchar *func_name = "modbus_udp_dac", *host;

    int port = 502;
    int debug = 0;
    int counter_value;
        
    modbus_packet_t     *pkt;    
    modbus_udp_handle_t handle;

    if (argc != 2)
    {
        printf("%s: usage: %s host \n", func_name, argv[0]);
        return 0;
    }
 
    host = argv[1];   

    if (modbus_udp_init(host, port, &handle) != 0)
    {
        printf("%s: modbus_udp_init failed.\n", __PRETTY_FUNCTION__);
        return 0;
    }
    
    // setup request packet
    pkt = modbus_packet_new();  

    modbus_read_holding_registers(pkt, 0x0009, 0x0001);

    if (debug)
    {
        modbus_packet_print(pkt);
    }

    // Send command
    if (modbus_udp_send(&handle, pkt) != 0)
    {
        printf("%s: modbus_udp_send failed.\n", __PRETTY_FUNCTION__);
        return 0;       
    }

    // recv response
    if (modbus_udp_recv(&handle, pkt) != 0)
    {
        printf("%s: modbus_udp_recv failed.\n", __PRETTY_FUNCTION__);
        return 0;       
    }
   
    if (debug)
    {
        modbus_packet_print(pkt);
    }

    counter_value = modbus_packet_reg32_get(pkt, 0);

    printf("COUNTER = %d\n", counter_value);

    if (debug)
    {
        printf("\n");
        printf("Response byte 1 0x%.2X\n", pkt->data_buff[0] & 0xFF);    
        printf("Response byte 2 0x%.2X\n", pkt->data_buff[1] & 0xFF);    
        printf("Response byte 3 0x%.2X\n", pkt->data_buff[2] & 0xFF);    
        printf("Response byte 4 0x%.2X\n", pkt->data_buff[3] & 0xFF);    
        printf("Response byte 5 0x%.2X\n", pkt->data_buff[4] & 0xFF);    
    }
    
    modbus_packet_free(pkt);

    return 0;
}



