//------------------------------------------------------------------------------
// Copyright (C) 2010, Raditex AB
// All rights reserved.
//
// FreeSCADA 
// http://www.FreeSCADA.com
// freescada@freescada.com
//
//------------------------------------------------------------------------------

#include <string.h>
#include <stdio.h>

#include "modbus.h"
#include "modbus-udp.h"


//------------------------------------------------------------------------------
// Execution starts here:
//------------------------------------------------------------------------------
int
main(int argc, char **argv)
{
    static char *func_name = "modbus_udp_digital_input", *host;

    int port = 502;
    int addr = 0, debug = 0;
        
    modbus_packet_t     *pkt;    
    modbus_udp_handle_t handle;

    if (argc != 3)
    {
        printf("%s: usage: %s host addr\n", func_name, argv[0]);
        printf("\taddr    =  1,2,3,4\n");
        return 0;
    }
 
    host = argv[1];   
    addr = atoi(argv[2]) - 1;

    if (modbus_udp_init(host, port, &handle) != 0)
    {
        printf("%s: modbus_udp_init failed.\n", __PRETTY_FUNCTION__);
        return 0;
    }
    
    // setup request packet
    pkt = modbus_packet_new();  

    modbus_read_input_status(pkt, 0x0000, 0x0004);
  
    if (debug)
    {
        modbus_packet_print(pkt);
    }

    // Send command
    if (modbus_udp_send(&handle, pkt) != 0)
    {
        printf("%s: modbus_udp_send failed.\n", __PRETTY_FUNCTION__);
        return 0;       
    }

    // recv response
    if (modbus_udp_recv(&handle, pkt) != 0)
    {
        printf("%s: modbus_udp_recv failed.\n", __PRETTY_FUNCTION__);
        return 0;       
    }
   
    if (debug)
    {
        modbus_packet_print(pkt);
    }

    printf("Digital input %d = %d\n", addr + 1, modbus_packet_data_bit_get(pkt, addr)); 

    if (debug)
    {
        printf("\nDEBUG: \n");
        printf("IO    1 = %d\n", (pkt->data_buff[1] & 0x01) ? 1 : 0);    
        printf("IO    2 = %d\n", (pkt->data_buff[1] & 0x02) ? 1 : 0);    
        printf("IO    3 = %d\n", (pkt->data_buff[1] & 0x04) ? 1 : 0);    
        printf("IO    4 = %d\n", (pkt->data_buff[1] & 0x08) ? 1 : 0);    
        printf("Relay 1 = %d\n", (pkt->data_buff[1] & 0x10) ? 1 : 0);    
        printf("Relay 1 = %d\n", (pkt->data_buff[1] & 0x20) ? 1 : 0);    
        printf("Relay 1 = %d\n", (pkt->data_buff[1] & 0x40) ? 1 : 0);    
        printf("Relay 1 = %d\n", (pkt->data_buff[1] & 0x80) ? 1 : 0);    
    }
     
    modbus_packet_free(pkt);

    return 0;
}



