//------------------------------------------------------------------------------
// Copyright (C) 2010, Raditex AB
// All rights reserved.
//
// FreeSCADA 
// http://www.FreeSCADA.com
// freescada@freescada.com
//
//------------------------------------------------------------------------------

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <glib.h>

#include "modbus.h"
#include "modbus-udp.h"

#define CMD_READ  0
#define CMD_WRITE 1

//------------------------------------------------------------------------------
// Execution starts here:
//------------------------------------------------------------------------------
int
main(int argc, char **argv)
{
    static gchar *func_name = "modbus_udp_digital_output", *host;

    int port = 502;
    int cmd = 0, addr = 0, value = 0, debug = 0;
        
    modbus_packet_t     *pkt;    
    modbus_udp_handle_t handle;

    if (argc < 4 || ((strcmp(argv[3], "read") != 0) && (strcmp(argv[3], "write") != 0)))
    {
        printf("%s: usage: %s host addr command [value]\n", func_name, argv[0]);
        printf("\taddr    =  1,2,3,4\n");
        printf("\tcommand = read | write value (value = 0 | 1)\n");
        return 0;
    }
 
    host = argv[1];   
    addr = atoi(argv[2]) - 1;
    cmd  = strcmp("write", argv[3]) == 0 ? CMD_WRITE : CMD_READ;
    if (argc == 5)
    {
        value = atoi(argv[4]);
    }
    
    // setup the udp socket
    if (modbus_udp_init(host, port, &handle) != 0)
    {
        printf("%s: modbus_udp_init failed.\n", __PRETTY_FUNCTION__);
        return 0;
    }

    // setup request packet
    pkt = modbus_packet_new();  
  
    if (cmd == CMD_READ)
    {   
        modbus_read_coil_status(pkt, 0x0000, 0x0004);

        //modbus_packet_set_transaction_id(pkt, 0x0BA1);
        //modbus_packet_set_function(pkt, MB_FUNC_READ_COIL_STATUS);
        //modbus_packet_set_unit(pkt, 0x00);
        //buff[offset++] = 0x00; // addr hi
        //buff[offset++] = 0x00; // addr lo
        //buff[offset++] = 0x00; // # regs hi
        //buff[offset++] = 0x04; // # regs lo
        //modbus_packet_set_data(pkt, buff, offset);
    }
    else
    {
        modbus_force_single_coil(pkt, addr, value);

        //modbus_packet_set_transaction_id(pkt, 0x13BE);
        //modbus_packet_set_function(pkt, MB_FUNC_FORCE_SINGLE_COIL);
        //modbus_packet_set_unit(pkt, 0x00);
        //buff[offset++] = 0x00; // addr hi
        //buff[offset++] = addr; // addr lo
        //buff[offset++] = value ? 0xFF : 0x00; // # regs hi
        //buff[offset++] = value ? 0xFF : 0x00; // # regs lo
        //modbus_packet_set_data(pkt, buff, offset);
    }

    if (debug)
    {
        modbus_packet_print(pkt);
    }

    // Send command
    if (modbus_udp_send(&handle, pkt) != 0)
    {
        printf("%s: modbus_udp_send failed.\n", __PRETTY_FUNCTION__);
        return 0;       
    }

    // recv response
    if (modbus_udp_recv(&handle, pkt) != 0)
    {
        printf("%s: modbus_udp_recv failed.\n", __PRETTY_FUNCTION__);
        return 0;       
    }
   
    if (debug)
    {
        modbus_packet_print(pkt);
    }

    if (cmd == CMD_READ)
    {
        printf("Digital output %d = %d\n", addr + 1, (pkt->data_buff[1] & (0x01<<addr)) ? 1 : 0);    

        if (debug)
        {
            printf("\nDEBUG: \n");
            printf("IO    1 = %d\n", (pkt->data_buff[1] & 0x01) ? 1 : 0);    
            printf("IO    2 = %d\n", (pkt->data_buff[1] & 0x02) ? 1 : 0);    
            printf("IO    3 = %d\n", (pkt->data_buff[1] & 0x04) ? 1 : 0);    
            printf("IO    4 = %d\n", (pkt->data_buff[1] & 0x08) ? 1 : 0);    
            printf("Relay 1 = %d\n", (pkt->data_buff[1] & 0x10) ? 1 : 0);    
            printf("Relay 1 = %d\n", (pkt->data_buff[1] & 0x20) ? 1 : 0);    
            printf("Relay 1 = %d\n", (pkt->data_buff[1] & 0x40) ? 1 : 0);    
            printf("Relay 1 = %d\n", (pkt->data_buff[1] & 0x80) ? 1 : 0);    
        }
    }
    

    modbus_packet_free(pkt);

    return 0;
}



