//------------------------------------------------------------------------------
// Copyright (C) 2010, Raditex AB
// All rights reserved.
//
// FreeSCADA 
// http://www.FreeSCADA.com
// freescada@freescada.com
//
//------------------------------------------------------------------------------

#include <unistd.h>
#include <string.h>
#include <stdio.h>


#include "modbus.h"
#include "modbus-udp.h"


//------------------------------------------------------------------------------
// Execution starts here:
//------------------------------------------------------------------------------
int
main(int argc, char **argv)
{
    static char *func_name = "modbus_udp_dac", *host;

    int port = 502;
    int cmd = 0, value = 0, debug = 1;
        
    modbus_packet_t     *pkt;    
    modbus_udp_handle_t handle;

    if (argc > 4 || ((strcmp(argv[2], "read") != 0) && (strcmp(argv[2], "write") != 0)))
    {
        printf("%s: usage: %s host command [value]\n", func_name, argv[0]);
        printf("\tcommand = read | write value (value = 0 - 255)\n");
        return 0;
    }
 
    host = argv[1];   
    cmd  = strcmp("write", argv[2]) == 0 ? 1 : 0;
    if (cmd == 1)
    {
//        value = atoi(argv[3]);
        value = strtol(argv[3], NULL, 16);
    }

    // init connection    
    if (modbus_udp_init(host, port, &handle) != 0)
    {
        printf("%s: modbus_udp_init failed.\n", __PRETTY_FUNCTION__);
        return 0;
    }

    // setup request packet
    pkt = modbus_packet_new();  
  
    if (cmd == 0)
    {   
        modbus_read_holding_registers(pkt, 0x000D, 0x0001);
    }
    else
    {
        modbus_preset_single_register(pkt, 0x000D, value);
    }

    if (debug)
    {
        modbus_packet_print(pkt);
    }

    // Send command
    if (modbus_udp_send(&handle, pkt) != 0)
    {
        printf("%s: modbus_udp_send failed.\n", __PRETTY_FUNCTION__);
        return 0;       
    }

    // recv response
    if (modbus_udp_recv(&handle, pkt) != 0)
    {
        printf("%s: modbus_udp_recv failed.\n", __PRETTY_FUNCTION__);
        return 0;       
    }
   
    if (debug)
    {
        modbus_packet_print(pkt);
    }

    if (cmd == 0)
    {
        value = modbus_packet_reg8_get(pkt, 0);
        printf("USART read = 0x%.2X\n", value & 0xFF);
    }
    
    if (debug)
    {
        printf("\n");
        printf("Response byte 1 0x%.2X\n", pkt->data_buff[0] & 0xFF);    
        printf("Response byte 2 0x%.2X\n", pkt->data_buff[1] & 0xFF);    
        printf("Response byte 3 0x%.2X\n", pkt->data_buff[2] & 0xFF);    
    }
    

    modbus_packet_free(pkt);

    return 0;
}



