//------------------------------------------------------------------------------
// Copyright (C) 2010, Raditex AB
// All rights reserved.
//
// FreeSCADA 
// http://www.FreeSCADA.com
// freescada@freescada.com
//
//------------------------------------------------------------------------------

#include <stdio.h>

#include "modbus.h"
#include "modbus-udp.h"

static char ipio_modbus_error_str[256];

//------------------------------------------------------------------------------
// read relay
//------------------------------------------------------------------------------
int
ipio_relay_read(char *host, int addr)
{
    int port = 502;
    int value = 0;
        
    modbus_packet_t     *pkt;    
    modbus_udp_handle_t handle;

    if (modbus_udp_init(host, port, &handle) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_init failed.\n", __PRETTY_FUNCTION__);
        return -1;
    }
    
    // setup request packet
    pkt = modbus_packet_new();  
    modbus_read_coil_status(pkt, 0x0000, 0x0004);

    // Send command
    if (modbus_udp_send(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_send failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return -1;       
    }

    // recv response
    if (modbus_udp_recv(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_recv failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return -1;       
    }
   
    value = (pkt->data_buff[1] & (0x10<<addr)) ? 1 : 0;

    modbus_packet_free(pkt);
    modbus_udp_close(&handle);

    return value;
}


//------------------------------------------------------------------------------
// write relay
//------------------------------------------------------------------------------
int
ipio_relay_write(char *host, int addr, int value)
{
    int port = 502;
        
    modbus_packet_t     *pkt;    
    modbus_udp_handle_t handle;

    if (modbus_udp_init(host, port, &handle) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_init failed.\n", __PRETTY_FUNCTION__);
        return -1;
    }
   
    // setup request packet
    pkt = modbus_packet_new();  
  
    modbus_force_single_coil(pkt, addr + 4, value);
 
    // Send command
    if (modbus_udp_send(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_send failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return -1;       
    }

    // recv response
    if (modbus_udp_recv(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_recv failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return -1;       
    }

    modbus_packet_free(pkt);
    modbus_udp_close(&handle);

    return 0;
}

//------------------------------------------------------------------------------
// read digital output
//------------------------------------------------------------------------------
int
ipio_digital_output_read(char *host, int addr)
{
    int port = 502;
    int value = 0;
        
    modbus_packet_t     *pkt;    
    modbus_udp_handle_t handle;
  
    // setup the udp socket
    if (modbus_udp_init(host, port, &handle) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_init failed.\n", __PRETTY_FUNCTION__);
        return -1;
    }

    // setup request packet
    pkt = modbus_packet_new();  
  
    modbus_read_coil_status(pkt, 0x0000, 0x0004);

    // Send command
    if (modbus_udp_send(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_send failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return -1;       
    }

    // recv response
    if (modbus_udp_recv(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_recv failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return -1;       
    }
   
    value = (pkt->data_buff[1] & (0x01<<addr)) ? 1 : 0;
    modbus_packet_free(pkt);
    modbus_udp_close(&handle);

    return value;
}


//------------------------------------------------------------------------------
// write digital output
//------------------------------------------------------------------------------
int
ipio_digital_output_write(char *host, int addr, int value)
{
    int port = 502;
        
    modbus_packet_t     *pkt;    
    modbus_udp_handle_t handle;
    
    // setup the udp socket
    if (modbus_udp_init(host, port, &handle) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_init failed.\n", __PRETTY_FUNCTION__);
        return 0;
    }

    // setup request packet
    pkt = modbus_packet_new();  

    modbus_force_single_coil(pkt, addr, value);

    // Send command
    if (modbus_udp_send(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_send failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return -1;       
    }

    // recv response
    if (modbus_udp_recv(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_recv failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return -1;  
    }  

    modbus_packet_free(pkt);
    modbus_udp_close(&handle);

    return 0;
}

//------------------------------------------------------------------------------
// read digital input
//------------------------------------------------------------------------------
int
ipio_digital_input_read(char *host, int addr)
{
    int port = 502;
    int value = 0;
        
    modbus_packet_t     *pkt;    
    modbus_udp_handle_t handle;

    if (modbus_udp_init(host, port, &handle) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_init failed.\n", __PRETTY_FUNCTION__);
        return 0;
    }
    
    // setup request packet
    pkt = modbus_packet_new();  
    modbus_read_input_status(pkt, 0x0000, 0x0004);
  
    // Send command
    if (modbus_udp_send(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_send failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return -1;       
    }

    // recv response
    if (modbus_udp_recv(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_recv failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return -1;       
    }
   
    value = modbus_packet_data_bit_get(pkt, addr);
    
    modbus_packet_free(pkt);
    modbus_udp_close(&handle);

    return value;
}

//------------------------------------------------------------------------------
// ipio dac read
//------------------------------------------------------------------------------
int
ipio_dac_read(char *host, int addr)
{
    int port = 502;
    int value = 0;
        
    modbus_packet_t     *pkt;    
    modbus_udp_handle_t handle;

    // init connection    
    if (modbus_udp_init(host, port, &handle) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_init failed.\n", __PRETTY_FUNCTION__);
        return -1;
    }

    // setup request packet
    pkt = modbus_packet_new();  
  
    modbus_read_holding_registers(pkt, addr, 0x0001);

    // Send command
    if (modbus_udp_send(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_send failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return -1;
    }

    // recv response
    if (modbus_udp_recv(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_recv failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return -1;
    }
   
    value = modbus_packet_reg16_get(pkt, 0);  

    modbus_packet_free(pkt);
    modbus_udp_close(&handle);

    return value;
}

//------------------------------------------------------------------------------
// ipio dac write
//------------------------------------------------------------------------------
int
ipio_dac_write(char *host, int addr, int value)
{
    int port = 502;
        
    modbus_packet_t     *pkt;    
    modbus_udp_handle_t handle;

    // init connection    
    if (modbus_udp_init(host, port, &handle) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_init failed.\n", __PRETTY_FUNCTION__);
        return -1;
    }

    // setup request packet
    pkt = modbus_packet_new();  
  
    modbus_preset_single_register(pkt, addr, value);

    // Send command
    if (modbus_udp_send(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_send failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return -1;
    }

    // recv response
    if (modbus_udp_recv(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_recv failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return -1;
    }
   
    modbus_packet_free(pkt);
    modbus_udp_close(&handle);

    return 0;
}

//------------------------------------------------------------------------------
// adc read
//------------------------------------------------------------------------------
int
ipio_adc_read(char *host, int addr)
{
    int port = 502;
    int value = 0;
        
    modbus_packet_t     *pkt;    
    modbus_udp_handle_t handle;

    if (modbus_udp_init(host, port, &handle) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_init failed.\n", __PRETTY_FUNCTION__);
        return -1;
    }
    
    // setup request packet
    pkt = modbus_packet_new();  

    modbus_read_holding_registers(pkt, addr+4, 0x0001);

    // Send command
    if (modbus_udp_send(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_send failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return -1;
    }

    // recv response
    if (modbus_udp_recv(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_recv failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return -1;
    }

    value = modbus_packet_reg16_get(pkt, 0);

    modbus_packet_free(pkt);
    modbus_udp_close(&handle);

    return value;
}



//------------------------------------------------------------------------------
// get firmware version
//------------------------------------------------------------------------------
int
ipio_version(char *host)
{   
    int version;
    int port = 502;
        
    modbus_packet_t     *pkt;    
    modbus_udp_handle_t handle;

    if (modbus_udp_init(host, port, &handle) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_init failed.\n", __PRETTY_FUNCTION__);
        return -1;
    }
    
    // setup request packet
    pkt = modbus_packet_new();  

    modbus_read_holding_registers(pkt, 0x000B, 0x0001);

    // Send command
    if (modbus_udp_send(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_send failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return -1;       
    }

    // recv response
    if (modbus_udp_recv(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_recv failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return -1;       
    }

    version = modbus_packet_reg8_get(pkt, 0);
   
    modbus_packet_free(pkt);
    modbus_udp_close(&handle);

    return version;
}

//------------------------------------------------------------------------------
// read counter
//------------------------------------------------------------------------------
int
ipio_counter_read(char *host)
{
    int port = 502;
    int value;
        
    modbus_packet_t     *pkt;    
    modbus_udp_handle_t handle;

    if (modbus_udp_init(host, port, &handle) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_init failed.\n", __PRETTY_FUNCTION__);
        return 0;
    }
    
    // setup request packet
    pkt = modbus_packet_new();  

    modbus_read_holding_registers(pkt, 0x0009, 0x0001);

    // Send command
    if (modbus_udp_send(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_send failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return 0;       
    }

    // recv response
    if (modbus_udp_recv(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_recv failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return 0;       
    }

    value = modbus_packet_reg32_get(pkt, 0);
   
    modbus_packet_free(pkt);
    modbus_udp_close(&handle);

    return value;
}

//------------------------------------------------------------------------------
// read temperature
//------------------------------------------------------------------------------
double
ipio_temperature_read(char *host)
{
    int port = 502;
    int value;
        
    modbus_packet_t     *pkt;    
    modbus_udp_handle_t handle;
 
    if (modbus_udp_init(host, port, &handle) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_init failed.\n", __PRETTY_FUNCTION__);
        return -120.0;
    }
    
    // setup request packet
    pkt = modbus_packet_new();  

    modbus_read_holding_registers(pkt, 0x000A, 0x0001);

    // Send command
    if (modbus_udp_send(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_send failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return -120.0;
    }

    // recv response
    if (modbus_udp_recv(&handle, pkt) != 0)
    {
        snprintf(ipio_modbus_error_str, sizeof(ipio_modbus_error_str), "%s: modbus_udp_recv failed.\n", __PRETTY_FUNCTION__);
        modbus_packet_free(pkt);
        modbus_udp_close(&handle);
        return -120.0;
    }
   
    value = modbus_packet_reg8_get(pkt, 0);
  
    modbus_packet_free(pkt);
    modbus_udp_close(&handle);

    return (double)value;
}


