//------------------------------------------------------------------------------
// Copyright (C) 2010, Raditex AB
// All rights reserved.
//
// FreeSCADA 
// http://www.FreeSCADA.com
// freescada@freescada.com
//
//------------------------------------------------------------------------------

//
// Setting the IP address of an IP I/O module using ARP packets
//


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <libnet.h>
#include <pcap.h>

#ifndef ETH_ALEN
#define ETH_ALEN 6
#endif

#ifndef IP_ALEN
#define IP_ALEN 4
#endif

static u_char eth_src[ETH_ALEN] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
static u_char eth_dst[ETH_ALEN] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

static u_char ip_src[IP_ALEN] = {192, 168, 0, 250};
static u_char ip_dst[IP_ALEN] = {192, 168, 0, 250};

void get_ip_addr(u_char *ip_addr,  char *str);
void get_hw_addr(u_char *mac_addr, char *str);

void
get_ip_addr(u_char *addr, char *str) 
{
    int i;
    struct in_addr in_addr;
    struct hostent *hostp;
    char *str_ip;    

    if ((hostp = gethostbyname(str)) != NULL)
   	{
        bcopy(hostp->h_addr, &in_addr, hostp->h_length);
    }
    else
    {
        in_addr.s_addr = inet_addr(str);
        if (in_addr.s_addr == INADDR_NONE)
        {
            fprintf(stderr,"Unknown host %s\n",str);
            exit(1);
        }
    }
    
    // no in_addr contains the IP address
    str_ip = inet_ntoa(in_addr);

    //printf("Resolved %s to %s\n", str, str_ip);

    for (i = 0; i < IP_ALEN; i++)
    {
        addr[i] = strtol(str_ip, &str_ip, 10);
        str_ip++;
    }

}

void
get_hw_addr(u_char *addr, char *str)
{
    int i;
    char c, val;

    for (i = 0; i < ETH_ALEN; i++)
    {
        addr[i] = strtol(str, &str, 16);
        str++;
    }

    return;

    for (i=0; i<ETH_ALEN; i++)
    {
	
	    if (!(c = tolower(*str++)))
	    {
	        fprintf(stderr, "Invalid MAC address: %s\n", str);
	        exit(1);
	    }

	    if (isdigit(c))
	    {
	        val = c-'0';
	    }
            else if (c >= 'a' && c <= 'f') 
	    {
	        val = c-'a'+10;
	    }
            else
	    {
	        fprintf(stderr, "Invalid MAC address: %s\n", str);
	        exit(1);
	    }

        *addr = val << 4;

	    if (!(c = tolower(*str++)))
	    { 
	        fprintf(stderr, "Invalid MAC address: %s\n", str);
	        exit(1);
	    }

        if(isdigit(c))
	    {
	        val = c-'0';
	    }
	    else if(c >= 'a' && c <= 'f') 
	    {
	        val = c-'a'+10;
	    }
        else 
	    {
	        fprintf(stderr, "Invalid MAC address: %s\n", str);
	        exit(1);
	    }
    
	    *addr++ |= val;
    
        if (*str == ':' || *str == '-')
	        str++;
    }
}


int
usage(char *program_name)
{
    printf("usage: %s interface mac-addr ip-addr\n", program_name);
    printf("\n\tAssign a new IP address to a IP I/O module.\n");
    printf("\n\tMAC address format: aa:bb:cc:dd:ee:ff\n");
    printf("\n\tIP  address format: ddd.ddd.ddd.ddd\n");
  	return -1;    
}

int
main(int argc, char **argv)
{
    char *interface;

    libnet_t *libnet = NULL;
    libnet_ptag_t arp=0, eth=0;
    char error[LIBNET_ERRBUF_SIZE];
    int i, c;

    if (argc != 4)
    {
        usage(argv[0]);
        return -1;
    }

    //
    // get the necessary permission to send arp packets
    //
    if (getuid() && geteuid())
    {
        fprintf(stderr, "%s: must be run as root\n", argv[0]);
        return -2;
    }

    //
    // Get IP/MAC addresses from cmd line
    //

    interface = argv[1];
    get_ip_addr(ip_src, argv[3]);
    get_ip_addr(ip_dst, argv[3]);

    get_hw_addr(eth_src, argv[2]);
    get_hw_addr(eth_dst, argv[2]);

    //
    // print addresses
    //
    printf("Sending request to unit with MAC address [on %s]: \n", interface);
    for (i = 0; i < ETH_ALEN-1; i++)
    {
        printf("%.2x:", (u_int8_t)eth_src[i]);
    }
    printf("%.2x\n\n", (u_int8_t)eth_src[ETH_ALEN-1]);

    printf("New IP address: \n");
    for (i = 0; i < IP_ALEN-1; i++)
    {
        printf("%d.", (u_int8_t)ip_src[i]);
    }
    printf("%d\n\n", (u_int8_t)ip_src[IP_ALEN-1]);

    //
    // build and set the ARP packet
    //
    if ((libnet = libnet_init(LIBNET_LINK, interface, error)) == NULL)
    {
        fprintf(stderr, "\nlibnet error: %s\n", error);
        return -3;
    }
    arp = libnet_build_arp(ARPHRD_ETHER, ETHERTYPE_IP, ETH_ALEN, IP_ALEN, ARPOP_REQUEST,
                           eth_src, ip_src, eth_dst, ip_dst, NULL, 0, libnet, arp);
    eth = libnet_build_ethernet(eth_dst, eth_src, ETHERTYPE_ARP, NULL, 0, libnet, eth);
    c = libnet_write(libnet);
    libnet_destroy(libnet);

    return c;
}

