//------------------------------------------------------------------------------
// Copyright (C) 2010, Raditex AB
// All rights reserved.
//
// FreeSCADA 
// http://www.FreeSCADA.com
// freescada@freescada.com
//
//------------------------------------------------------------------------------

#include <gtk/gtk.h>
#include <string.h>
#include <modbus.h>
#include <ipio-modbus.h>

static int connected = -1;
static char *host = NULL;

static GtkAdjustment *adj_dac1, *adj_dac2, *adj_dac3, *adj_dac4;
static GtkWidget *lbl_dac1, *lbl_dac2, *lbl_dac3, *lbl_dac4;
static GtkWidget *lbl_counter, *lbl_temperature;
static GtkAdjustment *adj_adc1, *adj_adc2, *adj_adc3, *adj_adc4;
static GtkWidget *lbl_adc1, *lbl_adc2, *lbl_adc3, *lbl_adc4;
static GtkWidget *relay1, *relay2, *relay3, *relay4;
static GtkWidget *dio1, *dio2, *dio3, *dio4;
#define DIOMODE_IN  0
#define DIOMODE_OUT 1
static int diomode1 = 0, diomode2 = 0, diomode3 = 0, diomode4 = 0;
static GtkWidget *combo, *sbar;
static GtkWidget *ipaddr, *macaddr;
static GList *host_list;
static guint relay_ignore = 0;
static GdkColor on  = {0, 60000, 30000, 30000};
static GdkColor off = {0, 30000, 60000, 30000};

static void setup_tree_view( GtkWidget *);
static void cell_edited(GtkCellRendererText *, gchar *, gchar *, GtkTreeView *);
static void cell_select(GtkCellRendererText *, gchar *, gchar *, GtkTreeView *);
//static void row_activated(GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *column, gpointer data);
static void row_activated(GtkTreeView *treeview, GtkTreePath *path,GtkTreeIter iter);
void on_changed(GtkWidget *widget, gpointer label);

// For the treeview
enum
{
   IP_NUMBER = 0,
   COLUMNS
};

typedef struct
{
   gchar *ip_number;
} IpItem;

const IpItem list[] =
{
   { ""},
   { NULL }
};


//----------------------------------------------------------------------
//
//----------------------------------------------------------------------
static void
button_set_color(GtkWidget *button, GdkColor color)
{
    return;

    int i;
	GtkStyle *style, *defstyle;

	defstyle = gtk_widget_get_default_style();

	style = gtk_style_copy(defstyle);

	for (i = 0; i < 5; i++)
    {
		style->bg[i] = color;
    }

	gtk_widget_set_style(button, style);
}

//----------------------------------------------------------------------
// Wrapper functions for menu creation.
//----------------------------------------------------------------------
GtkWidget *
create_menu_item(GtkWidget *menu, gchar *azlabel, GtkSignalFunc sfunc, gpointer *sfunc_data)
{
    GtkWidget *new_item;

    if (azlabel)
    {
        new_item = gtk_menu_item_new_with_label(azlabel);
        gtk_signal_connect(GTK_OBJECT(new_item), "activate",
		                   GTK_SIGNAL_FUNC(sfunc), sfunc_data);
    }
    else
    {
        new_item = gtk_menu_item_new();
    }

    /* add tooltip here */
    gtk_menu_append(GTK_MENU(menu), new_item);
    gtk_widget_show(new_item);

    return new_item;
}


GtkWidget *
create_sub_menu(GtkWidget *parent_menu, gchar *azlabel, gboolean rjust)
{
    GtkWidget *submenu, *item;

    item = gtk_menu_item_new_with_label(azlabel);
    gtk_menu_bar_append(GTK_MENU_BAR(parent_menu), item);

    submenu = gtk_menu_new();
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);

    if (rjust)
    {
        gtk_menu_item_right_justify(GTK_MENU_ITEM(item));
    }

    gtk_widget_show(item);
    gtk_widget_show(submenu);

    return submenu;
}

static void
send_stat_request()
{
    char buff[256];

    int rel1, rel2, rel3, rel4;
    int di1, di2, di3, di4;
    int do1, do2, do3, do4;
    int dac1, dac2, dac3, dac4;
    int adc1, adc2, adc3, adc4;
    int counter;
    double temperature;

    if (host == NULL || connected == -1)
        return;

    // relay
    if ((rel1 = ipio_relay_read(host, 0)) == -1)
        goto read_error;

    if ((rel2 = ipio_relay_read(host, 1)) == -1)
        goto read_error;

    if ((rel3 = ipio_relay_read(host, 2)) == -1)
        goto read_error;

    if ((rel4 = ipio_relay_read(host, 3)) == -1)
        goto read_error;


    // digital input
    if ((di1 = ipio_digital_input_read(host, 0)) == -1)
        goto read_error;

    if ((di2 = ipio_digital_input_read(host, 1)) == -1)
        goto read_error;

    if ((di3 = ipio_digital_input_read(host, 2)) == -1)
        goto read_error;

    if ((di4 = ipio_digital_input_read(host, 3)) == -1)
        goto read_error;

    // digital output
    if ((do1 = ipio_digital_output_read(host, 0)) == -1)
        goto read_error;

    if ((do2 = ipio_digital_output_read(host, 1)) == -1)
        goto read_error;

    if ((do3 = ipio_digital_output_read(host, 2)) == -1)
        goto read_error;

    if ((do4 = ipio_digital_output_read(host, 3)) == -1)
        goto read_error;


    // dac
    if ((dac1 = ipio_dac_read(host, 0)) == -1)
        goto read_error;

    if ((dac2 = ipio_dac_read(host, 1)) == -1)
        goto read_error;

    if ((dac3 = ipio_dac_read(host, 2)) == -1)
        goto read_error;

    if ((dac4 = ipio_dac_read(host, 3)) == -1)
        goto read_error;

    // adc
    if ((adc1 = ipio_adc_read(host, 0)) == -1)
        goto read_error;

    if ((adc2 = ipio_adc_read(host, 1)) == -1)
        goto read_error;

    if ((adc3 = ipio_adc_read(host, 2)) == -1)
        goto read_error;

    if ((adc4 = ipio_adc_read(host, 3)) == -1)
        goto read_error;

    if ((counter = ipio_counter_read(host)) == -1)
        goto read_error;

    if ((temperature = ipio_temperature_read(host)) == -120.0)
        goto read_error;

    // dac
    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj_dac1), 10.0 - 10.0/255 * dac1);
    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj_dac2), 10.0 - 10.0/255 * dac2);
    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj_dac3), 10.0 - 10.0/255 * dac3);
    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj_dac4), 10.0 - 10.0/255 * dac4);

    // adc
    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj_adc1), 10.0 - 10.0/255 * adc1);
    g_snprintf(buff, sizeof(buff), "%.2f", 10.0/255 * adc1);
    gtk_label_set_text(GTK_LABEL(lbl_adc1), buff);
    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj_adc2), 10.0 - 10.0/255 * adc2);
    g_snprintf(buff, sizeof(buff), "%.2f", 10.0/255 * adc2);
    gtk_label_set_text(GTK_LABEL(lbl_adc2), buff);
    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj_adc3), 10.0 - 10.0/255 * adc3);
    g_snprintf(buff, sizeof(buff), "%.2f", 10.0/255 * adc3);
    gtk_label_set_text(GTK_LABEL(lbl_adc3), buff);
    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj_adc4), 10.0 - 10.0/255 * adc4);
    g_snprintf(buff, sizeof(buff), "%.2f", 10.0/255 * adc4);
    gtk_label_set_text(GTK_LABEL(lbl_adc4), buff);

    if (diomode1 == DIOMODE_IN)
    {
        gtk_button_set_label(GTK_BUTTON(dio1), di1 ? "On" : "Off");
        // button_set_color(dio1, di1 ? on : off);
    }
    else
    {
        gtk_button_set_label(GTK_BUTTON(dio1), do1 ? "On" : "Off");
        // button_set_color(dio1, do1 ? on : off);
    }

    if (diomode2 == DIOMODE_IN)
    {
        gtk_button_set_label(GTK_BUTTON(dio2), di2 ? "On" : "Off");
        // button_set_color(dio2, di2 ? on : off);
    }
    else
    {
        gtk_button_set_label(GTK_BUTTON(dio2), do2 ? "On" : "Off");
        // button_set_color(dio2, do2 ? on : off);
    }

    if (diomode3 == DIOMODE_IN)
    {
        gtk_button_set_label(GTK_BUTTON(dio3), di3 ? "On" : "Off");
        // button_set_color(dio3, di3 ? on : off);
    }
    else
    {
        gtk_button_set_label(GTK_BUTTON(dio3), do3 ? "On" : "Off");
        //button_set_color(dio3, do3 ? on : off);
    }

    if (diomode4 == DIOMODE_IN)
    {
        gtk_button_set_label(GTK_BUTTON(dio4), di4 ? "On" : "Off");
        //button_set_color(dio4, di4 ? on : off);
    }
    else
    {
        gtk_button_set_label(GTK_BUTTON(dio4), do4 ? "On" : "Off");
        //button_set_color(dio4, do4 ? on : off);
    }

    // relays
    gtk_button_set_label(GTK_BUTTON(relay1), rel1 ? "On" : "Off");    
    //button_set_color(relay1, rel1 ? on : off);
    //gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(relay1), (rel1) ? TRUE : FALSE);

    gtk_button_set_label(GTK_BUTTON(relay2), rel2 ? "On" : "Off");    
    //button_set_color(relay2, rel2 ? on : off);
    //gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(relay2), (rel2) ? TRUE : FALSE);

    gtk_button_set_label(GTK_BUTTON(relay3), rel3 ? "On" : "Off");    
    //button_set_color(relay3, rel3 ? on : off);
    //gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(relay3), (rel3) ? TRUE : FALSE);

    gtk_button_set_label(GTK_BUTTON(relay4), rel4 ? "On" : "Off");    
    // button_set_color(relay4, rel4 ? on : off);
    // gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(relay4), (rel4) ? TRUE : FALSE);

    g_snprintf(buff, sizeof(buff), "%d", counter);
    gtk_label_set_text(GTK_LABEL(lbl_counter), buff);

    g_snprintf(buff, sizeof(buff), "%.2f C", temperature);
    gtk_label_set_text(GTK_LABEL(lbl_temperature), buff);

    gtk_statusbar_pop(GTK_STATUSBAR(sbar),  1);
    gtk_statusbar_push(GTK_STATUSBAR(sbar), 1, "");

    return;
read_error:
    gtk_statusbar_pop(GTK_STATUSBAR(sbar), 1);
    gtk_statusbar_push(GTK_STATUSBAR(sbar), 1, "Lost modbus UDP packet.");
    //printf("%s: IPIO MODBUS fail: %s.\n", __PRETTY_FUNCTION__, modbus_error_str);
    return;        
}

static gboolean
winipio_update(gpointer data)
{
    gtk_statusbar_push(GTK_STATUSBAR(sbar), 1, "updating");
    while (gtk_events_pending())
        gtk_main_iteration();

    send_stat_request();

    return TRUE;
}

static void
dac_change(GtkAdjustment *adjustment, gpointer user_data)
{
    int addr, value;
    gchar buff[128];

    sprintf(buff, "%.1f", 10.0-adjustment->value);

    if (adjustment == adj_dac1)
    {
        addr = 0x00;
        gtk_label_set_text(GTK_LABEL(lbl_dac1), buff);
    }
    if (adjustment == adj_dac2)
    {
        addr = 0x01;
        gtk_label_set_text(GTK_LABEL(lbl_dac2), buff);
    }
    if (adjustment == adj_dac3)
    {
        addr = 0x02;
        gtk_label_set_text(GTK_LABEL(lbl_dac3), buff);
    }
    if (adjustment == adj_dac4)
    {
        addr = 0x03;
        gtk_label_set_text(GTK_LABEL(lbl_dac4), buff);
    }

    if (!host || connected == -1)
        return;

    value = 255 * (1.0 - adjustment->value/10.0);

    if (ipio_dac_write(host, addr, value) == -1)
    {
        gtk_statusbar_pop(GTK_STATUSBAR(sbar), 1);
        gtk_statusbar_push(GTK_STATUSBAR(sbar), 1, "Lost UDP/dac packet.");
	    return;
    }

    gtk_statusbar_pop(GTK_STATUSBAR(sbar), 1);
    gtk_statusbar_push(GTK_STATUSBAR(sbar), 1, "");
}

static void
select_host(GtkEntry *entry, gpointer data)
{
    if (host == NULL || connected == -1)
        return;

   //host = (gchar *)gtk_entry_get_text(GTK_ENTRY(entry));
   // host =
   printf("Selected host: %s\n", host); 
}

static int
setip(const char *ip, const char *mac)
{
    char buff[256];

    g_snprintf(buff, sizeof(buff), "/usr/bin/gksudo /usr/local/freescada/bin/ipio_setip eth0 %s %s", mac, ip);

    return system(buff);
}

static void
winipio_setip(GtkButton *entry, gpointer data)
{
    const gchar *ip = gtk_entry_get_text(GTK_ENTRY(ipaddr));
    const gchar *mac = gtk_entry_get_text(GTK_ENTRY(macaddr));

    if (setip(ip, mac) == 0)
    {
        gtk_statusbar_pop(GTK_STATUSBAR(sbar), 1);
        gtk_statusbar_push(GTK_STATUSBAR(sbar), 1, "Set IP successful.");  
    }
    else
    {
        gtk_statusbar_pop(GTK_STATUSBAR(sbar), 1);
        gtk_statusbar_push(GTK_STATUSBAR(sbar), 1, "Failed to set new IP address.");  
    }
}

static gint
compare(gconstpointer a, gconstpointer b)
{
    return strcmp((char *)a, (char *)b);
}

static void
set_dio_mode()
{
/*
	const gchar *clbl;
	io_data io;
       
	clbl = gtk_button_get_label(GTK_BUTTON(dio1));
	io.out1 = (strcmp(clbl, "On") == 0) ? 1 : 0;
	io.out1 = (!diomode1) ? 1 : io.out1;

	clbl = gtk_button_get_label(GTK_BUTTON(dio2));
	io.out2 = (strcmp(clbl, "On") == 0) ? 1 : 0;
	io.out2 = (!diomode1) ? 1 : io.out2;

	clbl = gtk_button_get_label(GTK_BUTTON(dio3));
	io.out3 = (strcmp(clbl, "On") == 0) ? 1 : 0;
	io.out3 = (!diomode1) ? 1 : io.out3;

	clbl = gtk_button_get_label(GTK_BUTTON(dio4));
	io.out4 = (strcmp(clbl, "On") == 0) ? 1 : 0;
	io.out4 = (!diomode1) ? 1 : io.out4;

	
	if (!ioh) return;

	if (ipiomod_io_write(ioh, &io) == -1)
	{
		gtk_statusbar_pop(GTK_STATUSBAR(sbar), 1);
		gtk_statusbar_push(GTK_STATUSBAR(sbar), 1, "Couldn't configure DI/O.");
		return;
	}
*/
}

void
popup_message(gchar *message)
{
    GtkWidget *dialog, *lbl, *button, *sep, *image;
    gchar title[128];
    gchar msgbuff[512];

    g_snprintf(title,   sizeof (title), "WinIPIO");
    g_snprintf(msgbuff, sizeof (msgbuff), "      %s      ", message);
    
    dialog = gtk_dialog_new();
    lbl  = gtk_label_new(msgbuff);
    gtk_window_set_title(GTK_WINDOW(dialog), title);
    button = gtk_button_new_with_label("Close");
    g_signal_connect_object(G_OBJECT(button), "clicked", G_CALLBACK(gtk_widget_destroy), (gpointer)dialog, G_CONNECT_SWAPPED);

    sep = gtk_hseparator_new();
    gtk_widget_show(sep);
  
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), sep, FALSE, FALSE, 10);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), lbl, FALSE, FALSE, 15);
  
    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area), button);
    gtk_widget_show_all(dialog);
}

static void
get_version(GtkEntry *entry, gpointer data)
{
    char buff[256];
    int version;

    if (host == NULL || connected == -1)
        return;

    if ((version = ipio_version(host)) < 0)
    {
		gtk_statusbar_pop(GTK_STATUSBAR(sbar), 1);
		gtk_statusbar_push(GTK_STATUSBAR(sbar), 1, "Couldn't get firmware version");        
    }
    else
    {
        g_snprintf(buff, sizeof(buff), "Firmware version %d.", version);
        popup_message(buff);
    }
    
    return;
}

static void
connect_host(GtkEntry *entry, gpointer data)
{
    if (!host || strlen(host) < 1)
    {
        gtk_statusbar_pop(GTK_STATUSBAR(sbar), 1);
        gtk_statusbar_push(GTK_STATUSBAR(sbar), 1, "You must enter host address first!");
        return;
    }

    //if (g_list_find_custom(host_list, host, (GCompareFunc)compare) == NULL)
    //    host_list = g_list_prepend(host_list, (gpointer)g_strdup(host));

    //gtk_combo_set_popdown_strings(GTK_COMBO(combo), host_list);

    connected = 1;

    send_stat_request();

    set_dio_mode();
}

static void
rb_click_in(GtkWidget *widget, gpointer data)
{
    if (!data) return;
	*((int *)data) = 0;
	set_dio_mode();
	send_stat_request();
}

static void
rb_click_out(GtkWidget *widget, gpointer data)
{
    if (!data) return;
	*((int *)data) = 1;
	set_dio_mode();
	send_stat_request();
}

static void
dio_click(GtkWidget *widget, gpointer data)
{
	const char *clbl;
	//io_data io;
	int mode, addr = -1, value = -1;
	
	if (!host || connected == -1)  return;
	if (!data) return;

	mode = *((int *)data);

	if (mode == DIOMODE_OUT)
	{
        if (widget == dio1)
        {
            addr = 0x00;
        }
        if (widget == dio2)
        {
            addr = 0x01;
        }
        if (widget == dio3)
        {
            addr = 0x02;
        }
        if (widget == dio4)
        {
            addr = 0x03;
        }

		clbl = gtk_button_get_label(GTK_BUTTON(widget));
        value = (strcmp(clbl, "Off") == 0) ? 1 : 0;

		gtk_button_set_label(GTK_BUTTON(widget), (value) ? "On" : "Off");
		//button_set_color(widget, (value) ? on : off);

		if (ipio_digital_output_write(host, addr, value) == -1)
		{
			gtk_statusbar_pop(GTK_STATUSBAR(sbar), 1);
			gtk_statusbar_push(GTK_STATUSBAR(sbar), 1, "Lost UDP/io packet.");
			return;
		}
		gtk_statusbar_pop(GTK_STATUSBAR(sbar), 1);
		gtk_statusbar_push(GTK_STATUSBAR(sbar), 1, "");
	}
}

static void
relay_click(GtkWidget *widget, gpointer data)
{

   
   const char *clbl;
   int addr = -1, value = -1;
   
   if (host == NULL || connected == -1)
      return;
   
   if (widget == relay1)
   {
      addr = 0x00;
   }
    if (widget == relay2)
    {
        addr = 0x01;
    }
    if (widget == relay3)
    {
        addr = 0x02;
    }
    if (widget == relay4)
    {
        addr = 0x03;
    }
    
    clbl = gtk_button_get_label(GTK_BUTTON(widget));
    value = (strcmp(clbl, "Off") == 0) ? 1 : 0;

    gtk_button_set_label(GTK_BUTTON(widget), (value) ? "On" : "Off");
    //gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), (value) ? TRUE : FALSE);

    if (ipio_relay_write(host, addr, value) == -1)
    {
        gtk_statusbar_pop(GTK_STATUSBAR(sbar), 1);
        gtk_statusbar_push(GTK_STATUSBAR(sbar), 1, "Lost UDP/relay packet.");
        return;
    }

    gtk_statusbar_pop(GTK_STATUSBAR(sbar), 1);
    gtk_statusbar_push(GTK_STATUSBAR(sbar), 1, "");
}

//----------------------------------------------------------------------
//
//----------------------------------------------------------------------
static void
setup_tree_view(GtkWidget *treeview)
{
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;   
    GtkListStore *model;
    GtkTreeIter iter;
    GtkTreeSelection *selection; 

   
    // The model we want
    model = gtk_list_store_new(1, G_TYPE_STRING);

    // We want (for now) the firs entry blank so that
    // we got a select event when we make a choice
    //gtk_list_store_append(model, &iter);
    //gtk_list_store_set(model, &iter, 0, "", -1);
    gtk_list_store_append(model, &iter);
    gtk_list_store_set(model, &iter, 0, "192.168.0.250", -1);
    gtk_list_store_append(model, &iter);
    gtk_list_store_set(model, &iter, 0, "192.168.1.250", -1);
    gtk_list_store_append(model, &iter);
    gtk_list_store_set(model, &iter, 0, "192.168.1.1", -1);

    renderer = gtk_cell_renderer_combo_new();   
    g_object_set(renderer, "text-column", 0, "editable", TRUE, "has-entry", TRUE, "model", model, NULL);
   
    column = gtk_tree_view_column_new_with_attributes
        ("IP-number", renderer, "text", IP_NUMBER, NULL);
      
   gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
   
   //g_signal_connect(model,    "row-changed", G_CALLBACK(row_activated),     NULL);
   g_signal_connect(G_OBJECT(renderer), "edited",  G_CALLBACK(cell_edited), (gpointer) treeview);
   g_signal_connect(G_OBJECT(renderer), "changed", G_CALLBACK(cell_select), (gpointer) treeview);


   //selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
   //g_signal_connect(selection, "changed", G_CALLBACK(on_changed), NULL);
   
}

//----------------------------------------------------------------------
//
//----------------------------------------------------------------------
static void cell_edited(GtkCellRendererText *renderer,
			gchar *path,
			gchar *new_text,
			GtkTreeView *treeview)
{
    GtkTreeIter iter;
    GtkTreeModel *model;
   
   // Make sure text not empty
   if (g_ascii_strcasecmp(new_text,"") != 0)
   {
      model= gtk_tree_view_get_model(treeview);
      if( gtk_tree_model_get_iter_from_string(model, &iter, path))
	 gtk_list_store_set(GTK_LIST_STORE(model), &iter, IP_NUMBER, new_text, -1);

      printf("New text: %s\n", new_text);

      host = g_strdup(new_text);

   }
}

//----------------------------------------------------------------------
//
//----------------------------------------------------------------------
static void cell_select(GtkCellRendererText *renderer,
			gchar *path,
			gchar *new_text,
			GtkTreeView *treeview)
{
   GtkTreeIter iter;
   GtkTreeModel *model;
   //GtkTreePath *path;
   char *str_data, *value;
   GtkTreeSelection *selection;
   GValue        name;
   gchar        *name_string;


   //selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
   //if (gtk_tree_selection_get_selected(GTK_TREE_SELECTION(selection), &model, &iter))
   //{
   //     gtk_tree_model_get(model, &iter, 0, &value,  -1);
   //     g_print("cell_select: host = %s\n", value);
   //     g_free(value);
   // }

   //    gtk_tree_path_new_from_string       (const gchar *path);


   // FIX THIS
   
   // Make sure text not empty
   //if(g_ascii_strcasecmp(new_text,"") != 0)
   //{

   model= gtk_tree_view_get_model(treeview);
   gtk_tree_model_get_iter_from_string(model, &iter, path);

   //gtk_tree_model_get_iter( GTK_TREE_MODEL( model ), &iter, gtk_tree_path_new_from_string(path));

   memset( &name, 0, sizeof( name ) );
   gtk_tree_model_get_value( GTK_TREE_MODEL( model ), &iter, 0, &name );
   g_printf ("g_value: %s\n", g_value_get_string (&name));

   str_data = g_strdup_value_contents(&name);
   printf("Gvalue: %s\n", str_data);


   g_object_get(renderer, "text-column", &name_string, NULL);
   str_data = g_strdup(name_string);

   printf("Path: %s Iter: %s\n", path, name_string);
   
   //gtk_tree_model_get(model, &iter, 0, &str_data, -1);
   //gtk_tree_model_get(model, 0, 0, &str_data, -1);
   
   printf("str_data: %s\n", str_data);
   
   host = g_strdup(str_data);
   printf("host: %s\n", host );
   
   //}
}

//----------------------------------------------------------------------
//
//----------------------------------------------------------------------
void
on_changed(GtkWidget *widget, gpointer label) 
{
    GtkTreeIter iter;
    GtkTreeModel *model;
    char *value;

    if (gtk_tree_selection_get_selected(GTK_TREE_SELECTION(widget), &model, &iter))
    {
        gtk_tree_model_get(model, &iter, 0, &value,  -1);
        g_print("on_changed: host = %s\n", value);
        g_free(value);
    }
}

//----------------------------------------------------------------------
//
//----------------------------------------------------------------------
static void
row_activated(GtkTreeView *treeview,
              GtkTreePath *path,GtkTreeIter iter)
//              GtkTreeViewColumn *column,
  //            gpointer data)
{
    //GtkTreeIter iter;
    GtkTreeModel *model;
    gchar *host;

    model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
    
    gtk_tree_model_get_iter(model, &iter, path);
    
    gtk_tree_model_get(model, &iter, 0, &host, -1);

    g_message("row_activated: host = %s\n", host);

	g_free(host);
	
    return;
}


//----------------------------------------------------------------------
//
//   main
//
//----------------------------------------------------------------------
int
main(int argc, char *argv[])
{

   GdkColor true_color;
   GdkColor false_color;
   GdkColor prelight_color;

   GtkWidget *treeview;
   GtkListStore *store;
   GtkTreeIter iter;
   int i = 0;

    GtkWidget *window, *button, *mainvbox, *hbox, *vbox, *frame, *label,
              *sep, *subhbox, *subvbox, *menubar, *submenu, *notebook,
	          *nb_frame, *mvb2, *rb;


    gtk_init(&argc, &argv);
    
    // Alt 1 - faster than looking up color
    /*
      true_color.red = 65535;
      true_color.green = 0;
      true_color.blue = 0;
      
      false_color.red = 0;
      false_color.green = 65535;
      false_color.blue = 0;
    */
    
    // Alt 2 - somewhat SLOW!!!
    gdk_color_parse ("OrangeRed", &true_color);
    gdk_color_parse ("LightGreen", &false_color);
    gdk_color_parse ("SkyBlue", &prelight_color);
    
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(window), "WinIPIO");
    gtk_signal_connect(GTK_OBJECT (window), "destroy",
                       GTK_SIGNAL_FUNC (gtk_main_quit), NULL);

    mvb2 = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER (window), mvb2);
    gtk_container_set_border_width(GTK_CONTAINER(window), 0);

    menubar = gtk_menu_bar_new();
    gtk_box_pack_start(GTK_BOX(mvb2), menubar, FALSE, FALSE, 0);

    submenu = create_sub_menu(menubar, "File", FALSE);
    create_menu_item(submenu, "Exit", gtk_main_quit, NULL);

    notebook = gtk_notebook_new();
    mainvbox = gtk_vbox_new(FALSE, 0);
    nb_frame = gtk_frame_new(NULL);
    gtk_container_add(GTK_CONTAINER (nb_frame), mainvbox);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), nb_frame, 
    gtk_label_new("Operate"));

    gtk_box_pack_start(GTK_BOX(mvb2), notebook, FALSE, FALSE, 0);
 
    hbox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(mainvbox), hbox, FALSE, FALSE, 0);

    label = gtk_label_new("Address");
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);

    //combo = gtk_combo_new();

    //combo = gtk_combo_box_new_text();
    // gtk_combo_set_item_string(GTK_COMBO(combo), 1, "192.168.0.250");

    //gtk_combo_box_insert_text(GTK_COMBO(combo), 1, "192.168.0.250");
      
    treeview = gtk_tree_view_new(); 
    setup_tree_view(treeview);

    store = gtk_list_store_new(COLUMNS, G_TYPE_STRING);

    while( list[i].ip_number != NULL )
    {
       printf("%s\n", list[i].ip_number);

       gtk_list_store_append(store, &iter);
       gtk_list_store_set(store, &iter, IP_NUMBER, list[i].ip_number, -1);
       i++;
    }

    gtk_tree_view_set_model(GTK_TREE_VIEW(treeview), GTK_TREE_MODEL(store));
    g_object_unref(store);

    gtk_box_pack_start(GTK_BOX(hbox), treeview, TRUE, TRUE, 5);

    button = gtk_button_new_with_label("Connect");
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
                       GTK_SIGNAL_FUNC(connect_host), NULL);

    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 5);

    button = gtk_button_new_with_label("Firmware version");
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
                       GTK_SIGNAL_FUNC(get_version), NULL);
    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 5);

    sep = gtk_hseparator_new();
    gtk_box_pack_start(GTK_BOX(mainvbox), sep, FALSE, FALSE, 5);

    hbox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(mainvbox), hbox, TRUE, TRUE, 0);

    /* DI frame */
    frame = gtk_frame_new("DI/O");
    gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 0);
    subvbox = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(frame), subvbox);
    gtk_container_set_border_width(GTK_CONTAINER(frame), 5);
    gtk_container_set_border_width(GTK_CONTAINER(subvbox), 5);

    subhbox = gtk_hbox_new(FALSE, 0);
    label = gtk_label_new("DI/O 1");
    gtk_box_pack_start(GTK_BOX(subvbox), label, FALSE, FALSE, 2);
    gtk_box_pack_start(GTK_BOX(subvbox), subhbox, FALSE, FALSE, 2);
    rb = gtk_radio_button_new_with_label(NULL, "In");
    gtk_signal_connect(GTK_OBJECT(rb), "clicked", GTK_SIGNAL_FUNC(rb_click_in), &diomode1);
    gtk_box_pack_start(GTK_BOX(subhbox), rb, FALSE, FALSE, 2);
    rb = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(rb), "Out");
    gtk_signal_connect(GTK_OBJECT(rb), "clicked", GTK_SIGNAL_FUNC(rb_click_out), &diomode1);
    gtk_box_pack_start(GTK_BOX(subhbox), rb, FALSE, FALSE, 2);
    dio1 = gtk_button_new_with_label("On");
    gtk_signal_connect(GTK_OBJECT(dio1), "clicked", GTK_SIGNAL_FUNC(dio_click),  &diomode1); 
    gtk_widget_set_usize(GTK_WIDGET(dio1), 50, 25);
    gtk_box_pack_start(GTK_BOX(subhbox), dio1, TRUE, TRUE, 2);

    subhbox = gtk_hbox_new(FALSE, 0);
    label = gtk_label_new("DI/O 2");
    gtk_box_pack_start(GTK_BOX(subvbox), label, FALSE, FALSE, 2);
    gtk_box_pack_start(GTK_BOX(subvbox), subhbox, FALSE, FALSE, 2);
    rb = gtk_radio_button_new_with_label(NULL, "In");
    gtk_signal_connect(GTK_OBJECT(rb), "clicked", GTK_SIGNAL_FUNC(rb_click_in), &diomode2);
    gtk_box_pack_start(GTK_BOX(subhbox), rb, FALSE, FALSE, 2);
    rb = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(rb), "Out");
    gtk_signal_connect(GTK_OBJECT(rb), "clicked", GTK_SIGNAL_FUNC(rb_click_out), &diomode2);
    gtk_box_pack_start(GTK_BOX(subhbox), rb, FALSE, FALSE, 2);
    dio2 = gtk_button_new_with_label("On");
    gtk_signal_connect(GTK_OBJECT(dio2), "clicked", GTK_SIGNAL_FUNC(dio_click),  &diomode2);
    gtk_widget_set_usize(GTK_WIDGET(dio2), 50, 25);
    gtk_box_pack_start(GTK_BOX(subhbox), dio2, TRUE, TRUE, 2);

    subhbox = gtk_hbox_new(FALSE, 0);
    label = gtk_label_new("DI/O 3");
    gtk_box_pack_start(GTK_BOX(subvbox), label, FALSE, FALSE, 2);
    rb = gtk_radio_button_new_with_label(NULL, "In");
    gtk_signal_connect(GTK_OBJECT(rb), "clicked", GTK_SIGNAL_FUNC(rb_click_in), &diomode3);
    gtk_box_pack_start(GTK_BOX(subhbox), rb, FALSE, FALSE, 2);
    rb = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(rb), "Out");
    gtk_signal_connect(GTK_OBJECT(rb), "clicked", GTK_SIGNAL_FUNC(rb_click_out), &diomode3);
    gtk_box_pack_start(GTK_BOX(subhbox), rb, FALSE, FALSE, 2);
    gtk_box_pack_start(GTK_BOX(subvbox), subhbox, TRUE, FALSE, 2);
    dio3 = gtk_button_new_with_label("On");
    gtk_signal_connect(GTK_OBJECT(dio3), "clicked", GTK_SIGNAL_FUNC(dio_click),  &diomode3);
    gtk_widget_set_usize(GTK_WIDGET(dio3), 50, 25);
    gtk_box_pack_start(GTK_BOX(subhbox), dio3, TRUE, TRUE, 2);

    subhbox = gtk_hbox_new(FALSE, 0);
    label = gtk_label_new("DI/O 4");
    gtk_box_pack_start(GTK_BOX(subvbox), label, FALSE, FALSE, 2);
	rb = gtk_radio_button_new_with_label(NULL, "In");
    gtk_signal_connect(GTK_OBJECT(rb), "clicked", GTK_SIGNAL_FUNC(rb_click_in), &diomode4);
    gtk_box_pack_start(GTK_BOX(subhbox), rb, FALSE, FALSE, 2);
	rb = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(rb), "Out");
    gtk_signal_connect(GTK_OBJECT(rb), "clicked", GTK_SIGNAL_FUNC(rb_click_out), &diomode4);
    gtk_box_pack_start(GTK_BOX(subhbox), rb, FALSE, FALSE, 2);
    gtk_box_pack_start(GTK_BOX(subvbox), subhbox, TRUE, FALSE, 2);
    dio4 = gtk_button_new_with_label("On");
    gtk_signal_connect(GTK_OBJECT(dio4), "clicked", GTK_SIGNAL_FUNC(dio_click),  &diomode4);
    gtk_widget_set_usize(GTK_WIDGET(dio4), 50, 25);
    gtk_box_pack_start(GTK_BOX(subhbox), dio4, TRUE, TRUE, 2);
    /* relay frame */
    frame = gtk_frame_new("Relay");
    gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 0);
    subhbox = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(frame), subhbox);
    gtk_container_set_border_width(GTK_CONTAINER(frame), 5);
    gtk_container_set_border_width(GTK_CONTAINER(subhbox), 5);
    label = gtk_label_new("Relay 1");
    gtk_box_pack_start(GTK_BOX(subhbox), label, FALSE, FALSE, 2);

    relay1 = gtk_toggle_button_new_with_label("Off");
    gtk_widget_modify_bg(relay1, GTK_STATE_ACTIVE, &true_color);
    gtk_widget_modify_fg(relay1, GTK_STATE_ACTIVE, &true_color);
    gtk_widget_modify_base(relay1, GTK_STATE_ACTIVE, &true_color);

    gtk_widget_modify_bg(relay1, GTK_STATE_NORMAL, &false_color);
    gtk_widget_modify_fg(relay1, GTK_STATE_NORMAL, &false_color);
    gtk_widget_modify_base(relay1, GTK_STATE_NORMAL, &false_color);

    gtk_widget_modify_fg(relay1, GTK_STATE_PRELIGHT, &prelight_color);
    gtk_widget_modify_bg(relay1, GTK_STATE_PRELIGHT, &prelight_color);
    gtk_widget_modify_base(relay1, GTK_STATE_PRELIGHT, &prelight_color);

    gtk_signal_connect(GTK_OBJECT(relay1), "clicked",
                       GTK_SIGNAL_FUNC(relay_click), NULL);
    gtk_widget_set_usize(GTK_WIDGET(relay1), 50, 20);
    gtk_box_pack_start(GTK_BOX(subhbox), relay1, TRUE, TRUE, 2);

    label = gtk_label_new("Relay 2");
    gtk_box_pack_start(GTK_BOX(subhbox), label, FALSE, FALSE, 2);

    relay2 = gtk_toggle_button_new_with_label("Off");

    gtk_widget_modify_bg(relay2, GTK_STATE_ACTIVE, &true_color);
    gtk_widget_modify_fg(relay2, GTK_STATE_ACTIVE, &true_color);
    gtk_widget_modify_base(relay2, GTK_STATE_ACTIVE, &true_color);

    gtk_widget_modify_bg(relay2, GTK_STATE_NORMAL, &false_color);
    gtk_widget_modify_fg(relay2, GTK_STATE_NORMAL, &false_color);
    gtk_widget_modify_base(relay2, GTK_STATE_NORMAL, &false_color);

    gtk_widget_modify_fg(relay2, GTK_STATE_PRELIGHT, &prelight_color);
    gtk_widget_modify_bg(relay2, GTK_STATE_PRELIGHT, &prelight_color);
    gtk_widget_modify_base(relay2, GTK_STATE_PRELIGHT, &prelight_color);


    gtk_signal_connect(GTK_OBJECT(relay2), "clicked",
                       GTK_SIGNAL_FUNC(relay_click), NULL);
    gtk_widget_set_usize(GTK_WIDGET(relay2), 50, 20);
    gtk_box_pack_start(GTK_BOX(subhbox), relay2, TRUE, TRUE, 2);

    label = gtk_label_new("Relay 3");
    gtk_box_pack_start(GTK_BOX(subhbox), label, FALSE, FALSE, 2);

    relay3 = gtk_toggle_button_new_with_label("Off");
    gtk_widget_modify_bg(relay3, GTK_STATE_ACTIVE, &true_color);
    gtk_widget_modify_fg(relay3, GTK_STATE_ACTIVE, &true_color);
    gtk_widget_modify_base(relay3, GTK_STATE_ACTIVE, &true_color);

    gtk_widget_modify_bg(relay3, GTK_STATE_NORMAL, &false_color);
    gtk_widget_modify_fg(relay3, GTK_STATE_NORMAL, &false_color);
    gtk_widget_modify_base(relay3, GTK_STATE_NORMAL, &false_color);

    gtk_widget_modify_fg(relay3, GTK_STATE_PRELIGHT, &prelight_color);
    gtk_widget_modify_bg(relay3, GTK_STATE_PRELIGHT, &prelight_color);
    gtk_widget_modify_base(relay3, GTK_STATE_PRELIGHT, &prelight_color);



    gtk_signal_connect(GTK_OBJECT(relay3), "clicked",
                       GTK_SIGNAL_FUNC(relay_click), NULL);
    gtk_widget_set_usize(GTK_WIDGET(relay3), 50, 20);
    gtk_box_pack_start(GTK_BOX(subhbox), relay3, TRUE, TRUE, 2);

    label = gtk_label_new("Relay 4");
    gtk_box_pack_start(GTK_BOX(subhbox), label, FALSE, FALSE, 2);

    relay4 = gtk_toggle_button_new_with_label("Off");

    gtk_widget_modify_bg(relay4, GTK_STATE_ACTIVE, &true_color);
    gtk_widget_modify_fg(relay4, GTK_STATE_ACTIVE, &true_color);
    gtk_widget_modify_base(relay4, GTK_STATE_ACTIVE, &true_color);

    gtk_widget_modify_bg(relay4, GTK_STATE_NORMAL, &false_color);
    gtk_widget_modify_fg(relay4, GTK_STATE_NORMAL, &false_color);
    gtk_widget_modify_base(relay4, GTK_STATE_NORMAL, &false_color);

    gtk_widget_modify_fg(relay4, GTK_STATE_PRELIGHT, &prelight_color);
    gtk_widget_modify_bg(relay4, GTK_STATE_PRELIGHT, &prelight_color);
    gtk_widget_modify_base(relay4, GTK_STATE_PRELIGHT, &prelight_color);

    gtk_signal_connect(GTK_OBJECT(relay4), "clicked",
                       GTK_SIGNAL_FUNC(relay_click), NULL);
    gtk_widget_set_usize(GTK_WIDGET(relay4), 50, 20);
    gtk_box_pack_start(GTK_BOX(subhbox), relay4, TRUE, TRUE, 2);


    /* ADC frame */
    frame = gtk_frame_new("ADC");
    gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, TRUE, 0);
    subhbox = gtk_hbox_new(TRUE, 0);
    gtk_container_add(GTK_CONTAINER(frame), subhbox);
    gtk_container_set_border_width(GTK_CONTAINER(frame), 5);
    gtk_container_set_border_width(GTK_CONTAINER(subhbox), 5);

    subvbox = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(subhbox), subvbox, FALSE, FALSE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(subvbox), 5);

    label = gtk_label_new("ADC 1");
    gtk_box_pack_start(GTK_BOX(subvbox), label, FALSE, FALSE, 2);
    lbl_adc1 = gtk_label_new("0.0");
    gtk_box_pack_start(GTK_BOX(subvbox), lbl_adc1, FALSE, FALSE, 2);

    adj_adc1 = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 11.0, 0.01, 1.0, 1.0));
    button = gtk_vscale_new(GTK_ADJUSTMENT(adj_adc1));
    gtk_scale_set_draw_value(GTK_SCALE(button), FALSE);
    gtk_range_set_update_policy(GTK_RANGE(button), GTK_UPDATE_DELAYED);
    gtk_box_pack_start(GTK_BOX(subvbox), button, TRUE, TRUE, 2);

    subvbox = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(subhbox), subvbox, FALSE, FALSE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(subvbox), 5);

    label = gtk_label_new("ADC 2");
    gtk_box_pack_start(GTK_BOX(subvbox), label, FALSE, FALSE, 2);
    lbl_adc2 = gtk_label_new("0.0");
    gtk_box_pack_start(GTK_BOX(subvbox), lbl_adc2, FALSE, FALSE, 2);


    adj_adc2 = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 11.0, 0.01, 1.0, 1.0));
    button = gtk_vscale_new(GTK_ADJUSTMENT(adj_adc2));
    gtk_scale_set_draw_value(GTK_SCALE(button), FALSE);
    gtk_range_set_update_policy(GTK_RANGE(button), GTK_UPDATE_DELAYED);
    gtk_box_pack_start(GTK_BOX(subvbox), button, TRUE, TRUE, 2);

    subvbox = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(subhbox), subvbox, TRUE, FALSE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(subvbox), 5);

    label = gtk_label_new("ADC 2");
    gtk_box_pack_start(GTK_BOX(subvbox), label, FALSE, FALSE, 2);
    lbl_adc3 = gtk_label_new("0.0");
    gtk_box_pack_start(GTK_BOX(subvbox), lbl_adc3, FALSE, FALSE, 2);

    adj_adc3 = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 11.0, 0.01, 1.0, 1.0));
    button = gtk_vscale_new(GTK_ADJUSTMENT(adj_adc3));
    gtk_scale_set_draw_value(GTK_SCALE(button), FALSE);
    gtk_range_set_update_policy(GTK_RANGE(button), GTK_UPDATE_DELAYED);
    gtk_box_pack_start(GTK_BOX(subvbox), button, TRUE, TRUE, 2);

    subvbox = gtk_vbox_new(FALSE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(subvbox), 5);
    gtk_box_pack_start(GTK_BOX(subhbox), subvbox, FALSE, TRUE, 0);

    label = gtk_label_new("ADC 4");
    gtk_box_pack_start(GTK_BOX(subvbox), label, FALSE, FALSE, 2);
    lbl_adc4 = gtk_label_new("0.0");
    gtk_box_pack_start(GTK_BOX(subvbox), lbl_adc4, FALSE, FALSE, 2);

    adj_adc4 = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 11.0, 0.01, 1.0, 1.0));
    button = gtk_vscale_new(GTK_ADJUSTMENT(adj_adc4));
    gtk_scale_set_draw_value(GTK_SCALE(button), FALSE);
    gtk_range_set_update_policy(GTK_RANGE(button), GTK_UPDATE_DELAYED);
    gtk_box_pack_start(GTK_BOX(subvbox), button, TRUE, TRUE, 2);

//    sbar = gtk_statusbar_new();
//    gtk_box_pack_end(GTK_BOX(mvb2), sbar, FALSE, FALSE, 0);
//    gtk_statusbar_pop(GTK_STATUSBAR(sbar), 1);
//    gtk_statusbar_push(GTK_STATUSBAR(sbar), 1, "Ready");

    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj_adc1), 10.0);
    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj_adc2), 10.0);
    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj_adc3), 10.0);
    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj_adc4), 10.0);

    /* DAC frame */
    frame = gtk_frame_new("DAC");
    gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, TRUE, 0);
    subhbox = gtk_hbox_new(TRUE, 0);
    gtk_container_add(GTK_CONTAINER(frame), subhbox);
    gtk_container_set_border_width(GTK_CONTAINER(frame), 5);
    gtk_container_set_border_width(GTK_CONTAINER(subhbox), 5);

    subvbox = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(subhbox), subvbox, FALSE, FALSE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(subvbox), 5);

    label = gtk_label_new("DAC 1");
    gtk_box_pack_start(GTK_BOX(subvbox), label, FALSE, FALSE, 2);
    lbl_dac1 = gtk_label_new("0.0");
    gtk_box_pack_start(GTK_BOX(subvbox), lbl_dac1, FALSE, FALSE, 2);

    adj_dac1 = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 11.0, 0.01, 1.0, 1.0));
    gtk_signal_connect(GTK_OBJECT(adj_dac1), "value-changed",
                       GTK_SIGNAL_FUNC(dac_change), NULL);
    button = gtk_vscale_new(GTK_ADJUSTMENT(adj_dac1));
    gtk_scale_set_draw_value(GTK_SCALE(button), FALSE);
    gtk_range_set_update_policy(GTK_RANGE(button), GTK_UPDATE_DELAYED);
    //gtk_widget_set_usize(GTK_WIDGET(button), 10, 200);
    gtk_box_pack_start(GTK_BOX(subvbox), button, TRUE, TRUE, 2);

    subvbox = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(subhbox), subvbox, FALSE, FALSE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(subvbox), 5);

    label = gtk_label_new("DAC 2");
    gtk_box_pack_start(GTK_BOX(subvbox), label, FALSE, FALSE, 2);
    lbl_dac2 = gtk_label_new("0.0");
    gtk_box_pack_start(GTK_BOX(subvbox), lbl_dac2, FALSE, FALSE, 2);


    adj_dac2 = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 11.0, 0.01, 1.0, 1.0));
    gtk_signal_connect(GTK_OBJECT(adj_dac2), "value-changed",
                       GTK_SIGNAL_FUNC(dac_change), NULL);
    button = gtk_vscale_new(GTK_ADJUSTMENT(adj_dac2));
    gtk_scale_set_draw_value(GTK_SCALE(button), FALSE);
    gtk_range_set_update_policy(GTK_RANGE(button), GTK_UPDATE_DELAYED);
    //gtk_widget_set_usize(GTK_WIDGET(button), 200, 10);
    gtk_box_pack_start(GTK_BOX(subvbox), button, TRUE, TRUE, 2);

    subvbox = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(subhbox), subvbox, TRUE, FALSE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(subvbox), 5);

    label = gtk_label_new("DAC 2");
    gtk_box_pack_start(GTK_BOX(subvbox), label, FALSE, FALSE, 2);
    lbl_dac3 = gtk_label_new("0.0");
    gtk_box_pack_start(GTK_BOX(subvbox), lbl_dac3, FALSE, FALSE, 2);

    adj_dac3 = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 11.0, 0.01, 1.0, 1.0));
    gtk_signal_connect(GTK_OBJECT(adj_dac3), "value-changed",
                       GTK_SIGNAL_FUNC(dac_change), NULL);
    button = gtk_vscale_new(GTK_ADJUSTMENT(adj_dac3));
    gtk_scale_set_draw_value(GTK_SCALE(button), FALSE);
    gtk_range_set_update_policy(GTK_RANGE(button), GTK_UPDATE_DELAYED);
    //gtk_widget_set_usize(GTK_WIDGET(button), 200, 10);
    gtk_box_pack_start(GTK_BOX(subvbox), button, TRUE, TRUE, 2);

    subvbox = gtk_vbox_new(FALSE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(subvbox), 5);
    gtk_box_pack_start(GTK_BOX(subhbox), subvbox, FALSE, TRUE, 0);

    label = gtk_label_new("DAC 4");
    gtk_box_pack_start(GTK_BOX(subvbox), label, FALSE, FALSE, 2);
    lbl_dac4 = gtk_label_new("0.0");
    gtk_box_pack_start(GTK_BOX(subvbox), lbl_dac4, FALSE, FALSE, 2);

    adj_dac4 = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 11.0, 0.01, 1.0, 1.0));
    gtk_signal_connect(GTK_OBJECT(adj_dac4), "value-changed",
                       GTK_SIGNAL_FUNC(dac_change), NULL);
    button = gtk_vscale_new(GTK_ADJUSTMENT(adj_dac4));
    gtk_scale_set_draw_value(GTK_SCALE(button), FALSE);
    gtk_range_set_update_policy(GTK_RANGE(button), GTK_UPDATE_DELAYED);
    //gtk_widget_set_usize(GTK_WIDGET(button), 30, 200);
    gtk_box_pack_start(GTK_BOX(subvbox), button, TRUE, TRUE, 2);

    sbar = gtk_statusbar_new();
    gtk_box_pack_end(GTK_BOX(mvb2), sbar, FALSE, FALSE, 0);
    gtk_statusbar_pop(GTK_STATUSBAR(sbar), 1);
    gtk_statusbar_push(GTK_STATUSBAR(sbar), 1, "Ready");

    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj_dac1), 10.0);
    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj_dac2), 10.0);
    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj_dac3), 10.0);
    gtk_adjustment_set_value(GTK_ADJUSTMENT(adj_dac4), 10.0);

    /* temp and counter frame */
    subvbox = gtk_vbox_new(TRUE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(subvbox), 0);
    gtk_box_pack_start(GTK_BOX(hbox), subvbox, FALSE, TRUE, 0);
 
    frame = gtk_frame_new("Counter");
    gtk_box_pack_start(GTK_BOX(subvbox), frame, FALSE, TRUE, 0);
    lbl_counter = gtk_label_new("-");
    gtk_container_add(GTK_CONTAINER(frame), lbl_counter);
    gtk_container_set_border_width(GTK_CONTAINER(frame), 5);

    /* temp and counter frame */
    frame = gtk_frame_new("Temperature");
    gtk_box_pack_start(GTK_BOX(subvbox), frame, FALSE, TRUE, 0);
    lbl_temperature = gtk_label_new("-");
    gtk_container_add(GTK_CONTAINER(frame), lbl_temperature);
    gtk_container_set_border_width(GTK_CONTAINER(frame), 5);

    /* ARP TAB */
    mainvbox = gtk_vbox_new(FALSE, 0);
    nb_frame = gtk_frame_new(NULL);
    gtk_container_add(GTK_CONTAINER (nb_frame), mainvbox);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), 
    nb_frame, gtk_label_new("Set IP"));

    hbox = gtk_hbox_new(FALSE, 15);
    gtk_box_pack_start(GTK_BOX(mainvbox), hbox, FALSE, FALSE, 10);
    label = gtk_label_new(" Ip   Address");
    gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 50);
    ipaddr = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(hbox), ipaddr, TRUE, TRUE, 20);

    hbox = gtk_hbox_new(FALSE, 15);
    gtk_box_pack_start(GTK_BOX(mainvbox), hbox, FALSE, FALSE, 10);
    label = gtk_label_new("Mac Address");
    gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 50);
    macaddr = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(hbox), macaddr, TRUE, TRUE, 20);

    hbox = gtk_hbox_new(TRUE, 15);
    gtk_box_pack_start(GTK_BOX(mainvbox), hbox, FALSE, FALSE, 10);
    button = gtk_button_new_with_label("Apply");
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
                       GTK_SIGNAL_FUNC(winipio_setip), NULL);
    label = gtk_label_new("");
    gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 10);
    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 10);

    gtk_widget_show_all(window);

    gtk_timeout_add(2000, (GtkFunction)winipio_update, NULL);

    gtk_main();
    return(0);
}

